require 'erb'
require 'socket'
require 'rubygems/format'

# Adapt to the differences between rubygems < 1.0.0 and after
# Once we can be reasonably certain that everybody has version >= 1.0.0
# all this logic should be killed
GEM_VERSION = Gem::Version.create(Gem::RubyGemsVersion)
HAS_REMOTE_INSTALLER = GEM_VERSION < Gem::Version.create("1.0.0")

if HAS_REMOTE_INSTALLER
  require 'rubygems/remote_installer'
end

# Extend String with a word_wrap method, which we use in the ERB template
# below.  Taken with modification from the word_wrap method in ActionPack.
# Text::Format does the smae thing better.
class String
  def word_wrap(line_width = 80)
    gsub(/\n/, "\n\n").gsub(/(.{1,#{line_width}})(\s+|$)/, "\\1\n").strip
  end
end

module Gem
  class Requirement
    def rpm_version_transform(version)
      if version == "> 0.0.0" or version == ">= 0"
        version = ""
      end
      version
    end

    def to_rpm
      result = as_list
      return result.map { |version| rpm_version_transform(version) }
    end

  end
end

module Gem2Rpm
  Gem2Rpm::VERSION = "0.6.4"

  if HAS_REMOTE_INSTALLER
    def self.find_download_url(name, version)
      installer = Gem::RemoteInstaller.new
      dummy, download_path = installer.find_gem_to_install(name, "=#{version}")
      download_path += "/gems/" if download_path.to_s != ""
      return download_path
    end
  else
    def self.find_download_url(name, version)
      dep = Gem::Dependency.new(name, "=#{version}")
      fetcher = Gem::SpecFetcher.fetcher
      dummy, download_path = fetcher.find_matching(dep, false, false).first
      download_path += "gems/" if download_path.to_s != ""
      return download_path
    end
  end

  def Gem2Rpm.convert(fname, template=TEMPLATE, out=$stdout,
                      nongem=true, local=false)
    format = Gem::Format.from_file_by_path(fname)
    spec = format.spec
    spec.description ||= spec.summary
    download_path = "http://gems.rubyforge.org/gems/"
    unless local
      begin
        download_path = find_download_url(spec.name, spec.version)
      rescue Gem::Exception => e
        $stderr.puts "Warning: Could not retrieve full URL for #{spec.name}\nWarning: Edit the specfile and enter the full download URL as 'Source0' manually"
        $stderr.puts "#{e.inspect}"
      end
    end
    template = ERB.new(template, 0, '<>')
    out.puts template.result(binding)
  end

  # Returns the email address of the packager (i.e., the person running
  # gem2spec).  Taken from RPM macros if present, constructed from system
  # username and hostname otherwise.
  def Gem2Rpm.packager()
    packager = `rpm --eval '%{packager}'`.chomp
    if packager == '' or packager == '%{packager}'
      packager = "#{Etc::getpwnam(Etc::getlogin).gecos} <#{Etc::getlogin}@#{Socket::gethostname}>"
    end
    packager
  end

  TEMPLATE =
%q{# Generated from <%= File::basename(format.gem_path) %> by gem2rpm5 -*- rpm-spec -*-          
%define	rbname	<%= spec.name %>

Summary:	<%= spec.summary.gsub(/\.$/, "") %>
Name:		rubygem-%{rbname}

Version:	<%= spec.version %>
Release:	1
Group:		Development/Ruby
License:	GPLv2+ or Ruby
URL:		<%= spec.homepage %>
Source0:	<%= download_path %>%{rbname}-%{version}.gem
BuildRequires:	rubygems <% if spec.required_rubygems_version %><%= spec.required_rubygems_version.to_rpm %><% end %>
<% if spec.extensions.empty? %>
BuildArch:	noarch
<% else %>
BuildRequires:	ruby-devel
BuildRequires:	rubygem(rake)
<% end %>

%description
<%= spec.description.to_s.chomp.word_wrap(78) + "\n" %>

%package	doc
Summary:	Documentation for %{name}
Group:		Books/Computer books
Requires:	%{name} = %{EVRD}
<% if spec.extensions %>
BuildArch:	noarch
<% end %>

%description	doc
Documents<% if spec.has_rdoc %>, RDoc <% end %>& RI documentation for %{name}.

%prep
%setup -q

%build
%gem_build

%install
rm -rf %{buildroot}
%gem_install

%clean
rm -rf %{buildroot}

%files
<% for f in spec.executables.sort %>
%{_bindir}/<%= f %>
<% end %>
<% directories = [] %>
<% suffixes = [] %>
<% docfiles = [] %>
<% format.file_entries.each do |entry, data| %>
<%   path = entry['path'] %>
<%   dir = File.dirname(path) %>
<%   dir = (dir == "." ? "" : "/"+dir) %>
<%   unless directories.index(dir) %>
<%     directories.push(dir) %>
<%   end %>
<%   suffix = "%s/%s" % [dir, File.basename(path).gsub(/.*\./, "*.")] %>
<%   if spec.extra_rdoc_files.include?(path) %>
<%     unless docfiles.index(suffix) %>
<%       docfiles.push(suffix) %>
<%     end %>
<%   elsif !suffixes.index(suffix) %>
<%     suffixes.push(suffix) %>
<%   end %>
<% end %>
<% for d in directories.sort %>
%dir %{ruby_gemdir}/gems/%{rbname}-%{version}<%= d %>
<%   for s in suffixes.sort %>
<%     if s[0..s.rindex("/")-1] == d %>
%{ruby_gemdir}/gems/%{rbname}-%{version}<%= s %>
<%     end %>
<%   end %>
<% end %>
%{ruby_gemdir}/specifications/%{rbname}-%{version}.gemspec

%files doc
<% for d in docfiles.sort %>
%doc %{ruby_gemdir}/gems/%{rbname}-%{version}<%= d %>
<% end %><% if spec.has_rdoc %>
%doc %{ruby_gemdir}/doc/%{rbname}-%{version}
<% end %>

%changelog
* <%= Time.now.strftime("%a %b %d %Y") %> <%= packager %> <%= spec.version %>-1
- Initial package
}
end

# Local Variables:
# ruby-indent-level: 2
# End:
